/*
 *  list.c
 *  
 *
 *  Created by Ethan Phelps-Goodman on 7/9/05.
 *  Copyright 2005 __MyCompanyName__. All rights reserved.
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include "array.h"

const int ARRAY_SIZE = 100000;






// create a new empty array
ArrayDictionary new_array() {
	ArrayDictionary array = malloc( sizeof( KeyValuePair ) * ARRAY_SIZE );
	
	int i;
	for ( i = 0; i < ARRAY_SIZE; i++ ) {
		array[i].wordKey = 0;
		array[i].filesHead = 0;
	}
	
	return array;
}


// add 'file' to the list of files associated with 'word'
void insert_array( ArrayDictionary data, char* word, char* file ) {
	
	// find the word key, or the first non-null element in array
	int i = 0;
	while ( i < ARRAY_SIZE && data[i].wordKey != 0 
			&& strcmp( data[i].wordKey, word ) != 0 ) {
		i++;
	}
		
	// didn't find key and out of space, error.
	if ( i == ARRAY_SIZE ) {
		fprintf( stderr, "Too many strings inserted.\n" );
		return;
	}
	
	// the key was found, add file to the list
	if ( data[i].wordKey != 0 ) {
		// we're being sloppy here: since all words from a file
		// are added at once, then if this word has already appeared
		// in the file, then the file will be at the head of the list
		if ( strcmp( data[i].filesHead->string, file ) != 0 ) {
			StringList* l = malloc( sizeof(StringList) );
			l->string = file;
			l->next = data[i].filesHead;
			data[i].filesHead = l;
		}
		return;
	}
	
	// key wasn't found, add a new entry to array
	// create new memory for the key
	data[i].wordKey = malloc( strlen( word ) + 1 );
	strcpy( data[i].wordKey, word );
	data[i].filesHead = malloc( sizeof( StringList ) );
	data[i].filesHead->string = file;
	data[i].filesHead->next = 0;
	
}


// return the StringList associated with the word
StringList* find_array( ArrayDictionary data, char* word ) {
	
	// find the word key, or the first non-null element in array
	int i = 0;
	while ( i < ARRAY_SIZE && data[i].wordKey != 0 
			&& strcmp( data[i].wordKey, word ) != 0 ) {
		i++;
	}
		
	// didn't find key 
	if ( i == ARRAY_SIZE || data[i].wordKey == 0 ) {
		return 0;
	}

	// otherwise return the list
	return data[i].filesHead;
}




// simple test cases
void test_array() {

	ArrayDictionary dict = new_array();
	
	char* s1 = "the";
	char* s2 = "red";
	char* s3 = "dog";
	
	char* one = "one";
	char* two = "two";
	char* three = "three";
	
	// document one has all the words
	insert_array( dict, s1, one );
	insert_array( dict, s2, one );
	insert_array( dict, s3, one );

	// document two has two words
	insert_array( dict, s1, two );
	insert_array( dict, s2, two );
	
	// print stuff
	printStringList( find_array( dict, s1 ) );
	printStringList( find_array( dict, s2 ) );
	printStringList( find_array( dict, s3 ) );
	
	// document three
	insert_array( dict, s2, three );
	insert_array( dict, s3, three );

	// print stuff
	printStringList( find_array( dict, s1 ) );
	printStringList( find_array( dict, s2 ) );
	printStringList( find_array( dict, s3 ) );

}




